from typing import Literal

from ._base import (
    ForbidExtraBaseModel,
    NfcCardEnrollmentSessionId,
    NfcCardId,
    NfcCardToken,
    UserId,
)


# TODO: this should be a partial of User
class NfcCardUser(ForbidExtraBaseModel):
    id: UserId
    """Identity ID of the user."""
    first_name: str
    """First name of the user."""
    last_name: str
    """Last name of the user."""
    name: str
    """Full name of the user."""
    avatar: str
    """!!! note "not listed in API docs" """


class NfcCard(ForbidExtraBaseModel):
    token: NfcCardToken
    """Identity token of the NFC card."""
    display_id: NfcCardId
    """Display ID of the NFC card."""
    status: Literal["assigned", "pending", "disable", "deleted", "loss"]
    """Status of the NFC card."""
    alias: str
    """Preferred name of the NFC card."""
    card_type: str
    """Type of the NFC card."""
    user_id: Literal[""] | UserId
    """Owner ID of the NFC card."""
    user_type: Literal["USER", "VISITOR", ""]
    """Type of the owner."""
    user: NfcCardUser
    """Owner of the NFC card."""
    note: str
    """!!! note "not listed in API docs" """
    nfc_id: str | None = None
    """!!! note "not listed in API docs" """


class EnrollNfcCardResponse(ForbidExtraBaseModel):
    session_id: NfcCardEnrollmentSessionId
    """The session for enrolling an NFC card."""


class NfcCardEnrollmentStatus(ForbidExtraBaseModel):
    token: NfcCardToken
    """Unique NFC card token."""
    id: NfcCardId
    """Display ID of the NFC card.
    !!! note "documented as `card_id`"
    """
