"""3.1 User Schemas"""

from enum import StrEnum

from ._access_policy import AccessPolicy
from ._base import (
    ForbidExtraBaseModel,
    NfcCardId,
    NfcCardToken,
    PinCode,
    UnixTimestampDateTime,
    UserGroupId,
    UserId,
)


class UserStatus(StrEnum):
    ACTIVE = "ACTIVE"
    """The user account is in active status."""
    PENDING = "PENDING"
    """A new admin account has been invited by the SSO account, but the invitation has not been accepted."""
    DEACTIVATED = "DEACTIVATED"
    """The account has been deactivated."""


# TODO: should be a partial of NfcCard
class UserNfcCard(ForbidExtraBaseModel):
    id: NfcCardId
    """Display ID of the NFC card."""
    token: NfcCardToken
    """Unique NFC card token."""
    type: str
    """!!! note "not listed in API docs" """


class User(ForbidExtraBaseModel):
    id: UserId
    """Identity ID of the user."""
    first_name: str
    """First name of the user."""
    last_name: str
    """Last name of the user."""
    full_name: str
    """Full name of the user."""
    alias: str
    """Preferred name of the user."""
    user_email: str
    """Email of the user."""
    email: str
    """!!! note "not listed in API docs, Unclear how this is different from `user_email`" """
    email_status: str
    """The status of the user's email."""
    username: str
    """!!! note "not listed in API docs" """
    avatar_relative_path: str
    """!!! note "not listed in API docs" """
    phone: str
    """Contact phone number of the user."""
    employee_number: str
    """Employee number of the user."""
    onboard_time: UnixTimestampDateTime
    """User onboarding date."""
    status: UserStatus


class FullUser(User):
    nfc_cards: list[UserNfcCard]
    """Token associated with the bound NFC card."""
    pin_code: PinCode | None
    """Token associated with the bound PIN code."""
    access_policy_ids: list[str] | None = None
    """Collection of the access policy ID."""
    access_policies: list[AccessPolicy] | None = None
    """All policies assigned to the user."""


class UserGroup(ForbidExtraBaseModel):
    """!!! note "not listed in API docs" """

    id: UserGroupId
    full_name: str
    name: str
    up_id: UserGroupId
    up_ids: list[UserGroupId]
